'Filename   LCDCounter_108.bas
'Version    1.08
'Date       2013-09-18
'Purpose    60 MHz Frequency counter using 16x2 LCD
'Author     Georg Latzel, DL6GL
'Copyright  (c) Georg Latzel, free for private use under GNU license
'Hardware   ATtiny2313, Standard HD44780 16x2 LCD
'           74HC14 Schmitt trigger with preamplifier
'           74HC93 1/8 divider
'           NPN switch LSB/USB, base = lo or open: LSB, hi: USB
'           Jumper a  in: IF frequency  9.000 MHz
'           Jumper b  in: IF frequency 10.700 MHz
'           Jumper c  in: IF frequency added to counter frequency
'                    out: IF frequency subtracted from counter frequency
'           All jumpers out: counter frequency is shown
'Compiler   BASCOM-AVR 2.0.7.6
'           May be compiled with BASCOM trial version
'           For readable "camel case" variables mark "Don't change case"
'           in BASCOM Options-Programmer-Environment-Editor
'           Fuse bits can be set with AVR Studio 4.0 & ISP-PROG via USB
'           Flash used: 1,990 bytes (97%)
'Edit history
'V1.00      2007-08-30
'           first release
'V1.08      2013-09-18
'           xtal 16 MHz, improved timer handling

'==============================================================================
'FUSE-Bits (AVR Studio 4.0-Settings, [ ]=no check, [x]=check)
'In ATMEGA-Data sheets: Bit = 1 -> not set
'                       Bit = 0 -> set
'SELFPRGEN  [ ]
'DWEN       [ ]
'EESAVE     [ ]
'SPIEN      [x]
'WDTON      [ ]
'BODLEVEL   Brown-out detection at VCC=2.7V
'RSTDISBL   [ ]
'CKDIV8     [ ] deactivate standard [x]!!!
'CKOUT      [ ]
'SUT_CKSEL  Ext. Crystal Osc. High Freq., Start-up time 16k + 64ms

$regfile = "ATtiny2313.dat"                       'Controller ATtiny2313
$crystal = 16000000                               'xtal 16.000 MHz
$hwstack = 32
$swstack = 16
$framesize = 32

'Configure Port B ------------------------------------------------------------
'Port B: LCD + ISP
'     PB.0     Not used
'     PB.1     Not used
'     PB.2     LCD-RS
'     PB.3     LCD-E
'     PB.4     LCD-D4
'     PB.5     LCD-D5 + ISP(MOSI)
'     PB.6     LCD-D6 + ISP(MISO)
'     PB.7     LCD-D7 + ISP(SCK)
DDRB.0 = 0                                        'PB.0 = input
DDRB.1 = 0                                        'PB.1 = input
PORTB.0 = 1                                       'Pullup PB.0
PORTB.1 = 1                                       'Pullup PB.1
Config Lcd = 16 * 2
Config Lcdpin = Pin , Db7 = Portb.7 , Db6 = Portb.6 , Db5 = Portb.5
Config Lcdpin = Pin , Db4 = Portb.4 , Rs = Portb.2 , E = Portb.3
Config Lcdbus = 4

'Configure Port D ------------------------------------------------------------
'Port D: Jumper und Counter Input
'     PD.0     Not used
'     PD.1     =1: Show "LSB", =0: "USB", Base of T3 = low/high
'     PD.2     Jumper a: Offset ZF2 (9 MHz), wenn 0(gesteckt)
'     PD.3     Jumper b:Offset ZF3 (10.7 MHz alternativ), wenn 0(gesteckt)
'     PD.4     Jumper c: ZF addieren, wenn 0(gesteckt), sonst subtrahieren
'     PD.5     Counter Input
'     PD.6     Not used
Ddrd = &B00000000                                 'PD all input
Portd = &B01111111                                'Pullup Ports D0 to D6
Sideband Alias PIND.1
Jumper_a Alias PinD.2
Jumper_b Alias PinD.3
Jumper_c Alias PinD.4

'Configure Timer0 & Timer1 ---------------------------------------------------
'Configure Timer0-Interrupt for counter gate (Timer0 = 8bit) ------------------
'Overflow time: Overflow-Counts * Prescale / crystal frequency
'Possible Prescales: 8, 64, 256, 1024, here 256
'Chosen prescale: 64 for sufficient display multiplex frequency
'Gate time for Counter:
'Effective Timer0 Counts * 256 Timer0-Steps * 100 Timer0-overflows / Crystal freq.
'Shortening Timer0 gate time with PresetTimer0 will give the appropiate gate time
'Effective Timer0 Counts: 256 (8bit Timer0-Counts) - 56 (PresetTimer0) = 200
'Gives timer0 gate time
'for prescale = 256, presettimer = 56, timer0 runs = 100 (divide by 2^2)
'= 200 * 256 * 100 /  16.000.000 Hz = 0,32 sec
'for prescale = 256, presettimer = 56, timer0 runs = 200 (divide by 2^3)
'= 200 * 256 * 200 /  16.000.000 Hz = 0,64 sec
'for prescale = 1024, presettimer = 56, timer0 runs = 50 (divide by 2^3)
'= 200 * 1024 * 50 /  16.000.000 Hz = 0,64 sec
'Gives frequency in units of 100 Hz
'Config Timer0 = Timer , Prescale = 256
Config Timer0 = Timer , Prescale = 1024
Config Timer1 = Counter , Edge = Rising , Noise_Cancel = 1
Const Presettimer0 = 56                           'Initialize Timer0 (shorten time, >0)
Const bytTimer0_MaxRuns = 50                      'Max. timer0 runs
'Const bytTimer0_MaxRuns = 100                     'Max. timer0 runs
'Const bytTimer0_MaxRuns = 200                     'Max. timer0 runs

On Timer0 Timer0_isr                              'On Overflow go to Timer0_isr
On Timer1 Timer1_isr                              'On Overflow go to Timer1_isr

Dim dwdCount As DWord                             'Frequency counts
Dim wrdCountLo As Word At dwdCount Overlay        'Counts low word
Dim wrdCountHi As Word At dwdCount + 2 Overlay    'Counts high word
Dim strDis As String * 16                         'Display string
Dim bytDis(16) As Byte At strDis Overlay
Dim strFreq As String * 6
Dim bytFreq(6) As Byte At strFreq Overlay
Dim bytTimer0_runs As Byte                        'Timer0 overflow counter
Dim dwdTmp0 As DWord                              'Temp. variable

'intermediate frequency (IF) in Hz for frequency offset
Const lngZF1 = 9000000                            'IF Offset 1 (Hz)
Const lngZF2 = 10700000                           'IF Offset 2 (Hz)

Dim bytTmp0 As Byte
Dim bytTmp1 As Byte
Dim bytTmp2 As Byte

Cls
Cursor Off
Upperline
LCD "Counter V1.08"
Lowerline
Lcd "Hello from DL6GL"
Wait 1
Cls

bytTimer0_runs = 0
Timer0 = Presettimer0
Enable Timer0
SREG.7 = 1                                        'Enable all interrupts
Start Timer0


Do

Loop

'==============================================================================
Timer0_isr:
'Interrupt Service Routine for Timer0 (Counter gate)
'Set Timer0 preset time
'Increment timer0 runs until > max runs
'Stop Timer1 if max Timer0 runs are expired
'Get number of counts in gate Timer/Counter1 (wrdCountLo)
'wrdCountLo is the low word of overlayed dwdCount
'wrdCountHi,the high word of overlayed dwdCount, was incemented in Timer1_isr
'Divide dwdCount by 8 (right bit shift by 3, 2^3 = 8) = frequency, units 100 Hz
'Add or substract IF frequency if jumper c is in
'display frequeny (row 1), USB/LSB and IF offset (row 2)
'Clear wrdCountHi

   Timer0 = Presettimer0                          'Reset Timer0 start counts
   If bytTimer0_runs = 0 Then
      Enable Timer1
      Start TIMER1
      bytTimer0_runs = 1
   ElseIf bytTimer0_runs < bytTimer0_MaxRuns Then
      Incr bytTimer0_runs
   Else
      Stop Timer1
      wrdCountLo = Timer1                         'get low word (overlay dwdCount)
      bytTimer0_runs = 0
      Timer1 = 0

      Shift dwdCount , Right , 3                  'Divide by 8 (2^3)=frequency
      dwdTmp0 = 0                                 'Offset 0
      If Jumper_a = 0 Then                        'Jumper c set, Offset lngZF1
         dwdTmp0 = lngZF1
      Else
         If Jumper_b = 0 Then                     'Jumper d set, Offset lngZF2
            dwdTmp0 = lngZF2
         End If
      End If
      dwdTmp0 = dwdTmp0 / 100                     'Offset in units 100 Hz
      If Jumper_c = 1 Then                        'Jumper a not set: subtract IF
         dwdCount = dwdCount - dwdTmp0
      Else                                        'Jumper a is set: add IF
         dwdCount = dwdCount + dwdTmp0
      End If

      strDis = Space(16)                          'Prepare display line
      bytDis(3) = ","
      bytDis(7) = ","
      bytDis(10) = "k"
      bytDis(11) = "H"
      bytDis(12) = "z"

      strFreq = Str(dwdCount)
      strFreq = Format(strFreq , "     0")
      bytTmp1 = Len(strFreq)
      wrdCountHi = 0                              'Clear Timer1 overflow counts

      bytTmp2 = 0                                 'Format freq. string xx.xxx,x
      For bytTmp0 = 1 To bytTmp1
         bytTmp2 = bytTmp2 + 1
         If bytDis(bytTmp2) = 44 Then             'ASCII ","
            bytTmp2 = bytTmp2 + 1
         End If
         bytDis(bytTmp2) = bytFreq(bytTmp0)
      Next bytTmp0
      If bytDis(2) = 32 Then                      'ASCII Space
         bytDis(3) = 32                           'Erase leading comma
      Else
         bytDis(3) = 46                           'ASCII "."
      End If

      Upperline                                   '1. row: Frequency
      Lcd strDis
      Lowerline                                   '2. row: LSB/USB & IF-Offset
      If Sideband = 0 Then
         strFreq = "USB"
      Else
         strFreq = "LSB"
      End If
      dwdTmp0 = dwdTmp0 / 10                      'Offset in kHz
      strDis = Str(dwdTmp0)                       'ZF-Offset (kHz)
      strDis = Format(strDis , "    0")
      LCD strFreq ; " " ; "Offset " ; strDis
   End If

Return

'==============================================================================
Timer1_isr:
'Interrupt Service Routine for Timer1 (Counter)
'Increment counter high word. i.e. add 2^16 counts
   Incr wrdCountHi                                'Counter High Word
Return

'==============================================================================
End